@@ -1,7 +1,7 @@
+# SPDX-License-Identifier: MIT
+
 """
 Tests for PEP-526 type annotations.
-
-Python 3.6+ only.
 """
 
 import sys
@@ -92,7 +92,10 @@ class TestAnnotations:
         assert 1 == len(attr.fields(C))
         assert_init_annotations(C, x=typing.List[int])
 
-    @pytest.mark.parametrize("slots", [True, False])
+    @pytest.mark.skipif(
+        sys.version_info[:2] < (3, 11),
+        reason="Incompatible behavior on older Pythons",
+    )
     def test_auto_attribs(self, slots):
         """
         If *auto_attribs* is True, bare annotations are collected too.
@@ -111,7 +114,7 @@ class TestAnnotations:
         i = C(42)
         assert "C(a=42, x=[], y=2, z=3, foo=None)" == repr(i)
 
-        attr_names = set(a.name for a in C.__attrs_attrs__)
+        attr_names = {a.name for a in C.__attrs_attrs__}
         assert "a" in attr_names  # just double check that the set works
         assert "cls_var" not in attr_names
 
@@ -147,10 +150,9 @@ class TestAnnotations:
             x=typing.List[int],
             y=int,
             z=int,
-            foo=typing.Optional[typing.Any],
+            foo=typing.Any,
         )
 
-    @pytest.mark.parametrize("slots", [True, False])
     def test_auto_attribs_unannotated(self, slots):
         """
         Unannotated `attr.ib`s raise an error.
@@ -168,7 +170,6 @@ class TestAnnotations:
             "The following `attr.ib`s lack a type annotation: v, y.",
         ) == e.value.args
 
-    @pytest.mark.parametrize("slots", [True, False])
     def test_auto_attribs_subclassing(self, slots):
         """
         Attributes from base classes are inherited, it doesn't matter if the
@@ -249,7 +250,7 @@ class TestAnnotations:
 
     def test_nullary_converter(self):
         """
-        A coverter with no arguments doesn't cause a crash.
+        A converter with no arguments doesn't cause a crash.
         """
 
         def noop():
@@ -382,15 +383,15 @@ class TestAnnotations:
 
         assert attr.converters.optional(noop).__annotations__ == {}
 
-    @pytest.mark.xfail(
-        sys.version_info[:2] == (3, 6), reason="Does not work on 3.6."
+    @pytest.mark.skipif(
+        sys.version_info[:2] < (3, 11),
+        reason="Incompatible behavior on older Pythons",
     )
-    @pytest.mark.parametrize("slots", [True, False])
     def test_annotations_strings(self, slots):
         """
         String annotations are passed into __init__ as is.
 
-        It fails on 3.6 due to a bug in Python.
+        The strings keep changing between releases.
         """
         import typing as t
 
@@ -415,10 +416,9 @@ class TestAnnotations:
             x=typing.List[int],
             y=int,
             z=int,
-            foo=typing.Optional[typing.Any],
+            foo=typing.Any,
         )
 
-    @pytest.mark.parametrize("slots", [True, False])
     def test_typing_extensions_classvar(self, slots):
         """
         If ClassVar is coming from typing_extensions, it is recognized too.
@@ -516,7 +516,34 @@ class TestAnnotations:
         assert str is attr.fields(C).y.type
         assert None is attr.fields(C).z.type
 
-    @pytest.mark.parametrize("slots", [True, False])
+    @pytest.mark.skipif(
+        sys.version_info[:2] < (3, 9),
+        reason="Incompatible behavior on older Pythons",
+    )
+    def test_extra_resolve(self):
+        """
+        `get_type_hints` returns extra type hints.
+        """
+        from typing import Annotated
+
+        globals = {"Annotated": Annotated}
+
+        @attr.define
+        class C:
+            x: 'Annotated[float, "test"]'
+
+        attr.resolve_types(C, globals)
+
+        assert attr.fields(C).x.type == Annotated[float, "test"]
+
+        @attr.define
+        class D:
+            x: 'Annotated[float, "test"]'
+
+        attr.resolve_types(D, globals, include_extras=False)
+
+        assert attr.fields(D).x.type == float
+
     def test_resolve_types_auto_attrib(self, slots):
         """
         Types can be resolved even when strings are involved.
@@ -536,7 +563,6 @@ class TestAnnotations:
         assert typing.List[int] == attr.fields(A).b.type
         assert typing.List[int] == attr.fields(A).c.type
 
-    @pytest.mark.parametrize("slots", [True, False])
     def test_resolve_types_decorator(self, slots):
         """
         Types can be resolved using it as a decorator.
@@ -553,7 +579,6 @@ class TestAnnotations:
         assert typing.List[int] == attr.fields(A).b.type
         assert typing.List[int] == attr.fields(A).c.type
 
-    @pytest.mark.parametrize("slots", [True, False])
     def test_self_reference(self, slots):
         """
         References to self class using quotes can be resolved.
@@ -569,7 +594,6 @@ class TestAnnotations:
         assert A == attr.fields(A).a.type
         assert typing.Optional[A] == attr.fields(A).b.type
 
-    @pytest.mark.parametrize("slots", [True, False])
     def test_forward_reference(self, slots):
         """
         Forward references can be resolved.
@@ -618,6 +642,40 @@ class TestAnnotations:
         with pytest.raises(NameError):
             typing.get_type_hints(C.__init__)
 
+    def test_inheritance(self):
+        """
+        Subclasses can be resolved after the parent is resolved.
+        """
+
+        @attr.define()
+        class A:
+            n: "int"
+
+        @attr.define()
+        class B(A):
+            pass
+
+        attr.resolve_types(A)
+        attr.resolve_types(B)
+
+        assert int == attr.fields(A).n.type
+        assert int == attr.fields(B).n.type
+
+    def test_resolve_twice(self):
+        """
+        You can call resolve_types as many times as you like.
+        This test is here mostly for coverage.
+        """
+
+        @attr.define()
+        class A:
+            n: "int"
+
+        attr.resolve_types(A)
+        assert int == attr.fields(A).n.type
+        attr.resolve_types(A)
+        assert int == attr.fields(A).n.type
+
 
 @pytest.mark.parametrize(
     "annot",
